#!/usr/bin/env python3
#coding=utf-8

'''
EXTEND:

Using cxfreeze to create a lonote.exe file.

    Dependency: Windows platform, Python >= 3.2, cx_freeze

# How To Use:

    py setup.py build_exe

The build directory will be placed in "build_exe" folder.
'''

import os
import os.path
import sys
import platform
from distutils.core import setup
from distutils.command.build_scripts import build_scripts

# 擴充import路徑

def execDir():
    '''執行檔所在資料夾'''
    return os.path.abspath(os.path.dirname(sys.argv[0]))
    # return os.path.dirname(os.path.realpath(__file__))

# source pack library add
src_lib_dir = os.path.join(execDir(), 'lonotelib')
sys.path[0] = src_lib_dir

# 匯入lonote資料模組

import lonote.info

# 通用設定

garbage_target = os.path.join('share', 'lonote', 'garbage')

def collect_data_files():
    data_files = [
        ('share/applications', ['desktop/lonote.desktop']),
        ('share/pixmaps', ['desktop/lonote.png']),
        ('share/pixmaps', ['desktop/lonote.ico'])
        ]

    # server data
    for root_dir in ['server']:
        for dir, dirs, files in os.walk(root_dir):
            target = os.path.join('share', 'lonote', dir)
            if files:
                files = [os.path.join(dir, f) for f in files]
                data_files.append((target, files))

    # i18n
    for dir, dirs, files in os.walk('locale'):
        target = os.path.join('share', dir)
        if files:
            files = [os.path.join(dir, f) for f in files]
            data_files.append((target, files))

    # docs
    source_docdir = 'doc'
    for dir, dirs, files in os.walk(source_docdir):
        temp_dir = os.path.relpath(dir, source_docdir) # 裁掉第一層
        target = os.path.join('share', 'doc', 'lonote', temp_dir)
        if files:
            files = [os.path.join(dir, f) for f in files]
            data_files.append((target, files))
    doc_root = os.path.join('share', 'doc', 'lonote')
    data_files.append((doc_root, ['README.markdown']))

    # man
    unix_manual_dir = os.path.join('doc', 'man')
    for dir, dirs, files in os.walk(unix_manual_dir):
        subdir = os.path.relpath(dir, 'doc')
        target = os.path.join('share', subdir)
        if files:
            files = [os.path.join(dir, f) for f in files]
            data_files.append((target, files))

    # config
    data_files.append((garbage_target, [
                'config/lonote/book/.hgempty',
                'config/lonote/index/.hgempty',
                'config/lonote/static/.hgempty']))

    return data_files

class postprocess_build_scripts(build_scripts):
    '''在 POSIX 中，lonote.py 重命名至 lonote'''
    def run(self):
        build_scripts.run(self)
        if os.name == 'posix' and not self.dry_run:
            file = os.path.join(self.build_dir, 'lonote.py')
            print('renaming %s to %s' % (file, file[:-3]))
            os.rename(file, file[:-3]) # len('.py') == 3

setup_parms = {
        # basic info
        'name': 'lonote',
        'version': lonote.info.version,
        'description': lonote.info.desc_long,
        'author': lonote.info.designer,
        'author_email': lonote.info.designer_mail,
        'url': lonote.info.homepage,
        'download_url': lonote.info.download_url,
        'platforms': ['windows', 'linux'],
        'classifiers': [
            'Environment :: Web Environment',
            'Intended Audience :: End Users/Desktop',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: JavaScript',
            'Topic :: Internet :: WWW/HTTP :: WSGI',
            'Topic :: Text Editors' ],
        # files configure
        'requires': [
            'lxml', # using in diff function, not really require.
            # 'bottle (>=0.11)',
            # 'diff-match-patch (>=20120106)',
            # 'whoosh (>=2.4.1)',
            ],
        'scripts': ['lonote.py'],
        'py_modules': [
            'lonotelib.diff_match_patch',
            'lonotelib.bottle'],
        'package_dir': {'lonotelib': 'lonotelib'},
        'packages': [
            'lonotelib.lonote',
            'lonotelib.civa',
            'lonotelib.whoosh',
            'lonotelib.whoosh.codec',
            'lonotelib.whoosh.filedb',
            'lonotelib.whoosh.lang',
            'lonotelib.whoosh.matching',
            'lonotelib.whoosh.qparser',
            'lonotelib.whoosh.query',
            'lonotelib.whoosh.support'],
        'data_files': collect_data_files(),
        # extend process
        'cmdclass': { 'build_scripts': postprocess_build_scripts, },
        }

# Windows 設定

def windows_build_exe_dir():
    '''取得 windows build_exe dir'''
    def windows_platform_text():
        '''取得 windows 平台名稱'''
        bits, linkage = platform.architecture()
        bit = ''
        if platform.system() == 'Windows':
            if bits.startswith('32'):
                bit = 'win32'
            elif bits.startswith('64'):
                bit = 'win64'
        return bit

    build_dir = "build_exe"
    exe_dir = "lonote-{ver}-{bit}".format(
            ver = lonote.info.version,
            bit = windows_platform_text())
    build_exe_dir = os.path.join(build_dir, exe_dir)
    return build_exe_dir

def build_exe_setting():
    '''追加 build_exe 所用的相關資料'''

    build_exe_options = {
        "build_exe": windows_build_exe_dir(),
        "packages": [
            "os",
            'urllib.request',
            'decimal',
            'lxml', # 比較功能，建議有但可以不要
            'lonote.translate'],
        "include_files": [
            'server',
            'config',
            'doc',
            'locale'],
        "icon": os.path.join("desktop", "lonote.ico"),
        "include_msvcr": True}

    # EXTEND setup_parms
    setup_parms['options'] = {"build_exe": build_exe_options}
    setup_parms['executables'] = [
            Executable("lonote.py", base='Win32GUI')]

if platform.system() == 'Windows':
    try:
        # 嘗試加入 cxfreeze 相關功能
        from cx_Freeze import setup, Executable
        build_exe_setting()
    except ImportError:
        pass


# run setup
setup(**setup_parms)
